"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const error_1 = __importDefault(require("@pnpm/error"));
const lifecycle_1 = __importStar(require("@pnpm/lifecycle"));
const logger_1 = __importDefault(require("@pnpm/logger"));
const sort_packages_1 = __importDefault(require("@pnpm/sort-packages"));
const p_limit_1 = __importDefault(require("p-limit"));
const realpath_missing_1 = __importDefault(require("realpath-missing"));
const existsInDir_1 = __importDefault(require("./existsInDir"));
exports.default = async (params, opts) => {
    const [scriptName, ...passedThruArgs] = params;
    if (!scriptName) {
        throw new error_1.default('SCRIPT_NAME_IS_REQUIRED', 'You must specify the script you want to run');
    }
    let hasCommand = 0;
    const sortedPackageChunks = opts.sort
        ? (0, sort_packages_1.default)(opts.selectedProjectsGraph)
        : [Object.keys(opts.selectedProjectsGraph).sort()];
    const packageChunks = opts.reverse ? sortedPackageChunks.reverse() : sortedPackageChunks;
    const result = {
        fails: [],
        passes: 0,
    };
    const limitRun = (0, p_limit_1.default)(opts.workspaceConcurrency ?? 4);
    const stdio = !opts.stream &&
        (opts.workspaceConcurrency === 1 ||
            (packageChunks.length === 1 && packageChunks[0].length === 1))
        ? 'inherit'
        : 'pipe';
    const existsPnp = existsInDir_1.default.bind(null, '.pnp.cjs');
    const workspacePnpPath = opts.workspaceDir && await existsPnp(opts.workspaceDir);
    for (const chunk of packageChunks) {
        await Promise.all(chunk.map(async (prefix) => limitRun(async () => {
            const pkg = opts.selectedProjectsGraph[prefix];
            if (!pkg.package.manifest.scripts?.[scriptName] ||
                process.env.npm_lifecycle_event === scriptName &&
                    process.env.PNPM_SCRIPT_SRC_DIR === prefix) {
                return;
            }
            hasCommand++;
            try {
                const lifecycleOpts = {
                    depPath: prefix,
                    extraBinPaths: opts.extraBinPaths,
                    pkgRoot: prefix,
                    rawConfig: opts.rawConfig,
                    rootModulesDir: await (0, realpath_missing_1.default)(path_1.default.join(prefix, 'node_modules')),
                    scriptsPrependNodePath: opts.scriptsPrependNodePath,
                    scriptShell: opts.scriptShell,
                    shellEmulator: opts.shellEmulator,
                    stdio,
                    unsafePerm: true, // when running scripts explicitly, assume that they're trusted.
                };
                const pnpPath = workspacePnpPath ?? await existsPnp(prefix);
                if (pnpPath) {
                    lifecycleOpts.extraEnv = (0, lifecycle_1.makeNodeRequireOption)(pnpPath);
                }
                if (opts.enablePrePostScripts &&
                    pkg.package.manifest.scripts?.[`pre${scriptName}`] &&
                    !pkg.package.manifest.scripts[scriptName].includes(`pre${scriptName}`)) {
                    await (0, lifecycle_1.default)(`pre${scriptName}`, pkg.package.manifest, lifecycleOpts);
                }
                await (0, lifecycle_1.default)(scriptName, pkg.package.manifest, { ...lifecycleOpts, args: passedThruArgs });
                if (opts.enablePrePostScripts &&
                    pkg.package.manifest.scripts?.[`post${scriptName}`] &&
                    !pkg.package.manifest.scripts[scriptName].includes(`post${scriptName}`)) {
                    await (0, lifecycle_1.default)(`post${scriptName}`, pkg.package.manifest, lifecycleOpts);
                }
                result.passes++;
            }
            catch (err) { // eslint-disable-line
                logger_1.default.info(err);
                if (!opts.bail) {
                    result.fails.push({
                        error: err,
                        message: err.message,
                        prefix,
                    });
                    return;
                }
                /* eslint-disable @typescript-eslint/dot-notation */
                err['code'] = 'ERR_PNPM_RECURSIVE_RUN_FIRST_FAIL';
                err['prefix'] = prefix;
                /* eslint-enable @typescript-eslint/dot-notation */
                throw err;
            }
        })));
    }
    if (scriptName !== 'test' && !hasCommand && !opts.ifPresent) {
        const allPackagesAreSelected = Object.keys(opts.selectedProjectsGraph).length === opts.allProjects.length;
        if (allPackagesAreSelected) {
            throw new error_1.default('RECURSIVE_RUN_NO_SCRIPT', `None of the packages has a "${scriptName}" script`);
        }
        else {
            logger_1.default.info({
                message: `None of the selected packages has a "${scriptName}" script`,
                prefix: opts.workspaceDir,
            });
        }
    }
    (0, cli_utils_1.throwOnCommandFail)('pnpm recursive run', result);
};
//# sourceMappingURL=runRecursive.js.map